package gov.va.nvap.server.endpoint.adapter.announce.interceptor;

import gov.va.nvap.common.interceptor.Interceptor;
import gov.va.nvap.common.interceptor.InterceptorException;
import gov.va.nvap.common.transformer.TransformerException;
import gov.va.nvap.service.audit.AuditException;
import gov.va.nvap.service.audit.AuditService;
import gov.va.nvap.service.audit.data.ConsentAudit;

import org.springframework.beans.factory.annotation.Required;

/**
 * Intercepts the announce message for auditing purposes.
 * 
 * @author Asha Amritraj
 * 
 */
public class PatientAnnouncerAuditInterceptor implements
		Interceptor<Object, Object> {
	/**
	 * Audits the consent object.
	 */
	private AuditService auditService;
	/**
	 * Transformer to transform the announce message to ConsentAudit.
	 */
	private AnnounceToConsentAudit transformer;

	@Override
	public Object intercept(final Object announcePatientType)
			throws InterceptorException {

		try {
			// Transform
			final ConsentAudit audit = this.transformer
					.transform(announcePatientType);
			// Audit
			this.auditService.auditConsent(audit);
		} catch (final TransformerException ex) {
			throw new InterceptorException(ex);
		} catch (final AuditException ex) {
			throw new InterceptorException(ex);
		}
		return announcePatientType;
	}

	@Required
	public void setAuditService(final AuditService auditService) {
		this.auditService = auditService;
	}

	@Required
	public void setTransformer(final AnnounceToConsentAudit transformer) {
		this.transformer = transformer;
	}
}
