/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.nvap.web.dao;

import gov.va.nvap.web.facility.Facility;
import gov.va.nvap.web.facility.FacilityInfo;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author Johann Sonnenberg
 */
public class FacilityDAO {
    
    @PersistenceContext
    private EntityManager em;

    final String facilityInfoQuery = "SELECT f.FACILITY_ID, f.FACILITY_STATION, f.FACILITY_DNS, f.FACILITY_NAME, f.ALLOWED, f.VISN_ID, "
            + "f.FACILITY_PHONE, f.FACILITY_ADDRESS, f.FACILITY_CITY, f.FACILITY_STATE, f.FACILITY_POSTAL_CODE, f.FACILITY_PARENT, f.ACTIVE, v.VISN_NAME AS VISN_NAME, "
            + "(SELECT LISTAGG(inFac.FACILITY_NAME||'('||inFac.FACILITY_STATION||')', '|') WITHIN GROUP (ORDER BY FACILITY_NAME) " 
            + "FROM FACILITY inFac "
            + "WHERE inFac.FACILITY_PARENT = f.FACILITY_STATION) AS CHILDREN, " 
            + "fac.FACILITY_NAME AS PARENT_NAME " 
            + "FROM FACILITY f "
            + "LEFT JOIN VISN v " 
            + "ON f.VISN_ID = v.VISN_NUMBER " 
            + "LEFT JOIN FACILITY fac " 
            + "ON f.FACILITY_PARENT = fac.FACILITY_STATION ";
    
    final String whereClause = "WHERE f.FACILITY_STATION = :stationNumber ";
    
    public void setEntityManager(EntityManager entityManager) {
        this.em = entityManager;
    }
    
    public List<Facility> findAll() {
        try {
            final TypedQuery<Facility> q = this.em.createNamedQuery("Facility.findAll", Facility.class);
            return q.getResultList();
        } catch (final NoResultException nre) {
            return new ArrayList<Facility>();
        }
    }
    
    public List<Facility> findAllSortByVisn() {
        try {
            final TypedQuery<Facility> q = this.em.createNamedQuery("Facility.findAllSortByVisn", Facility.class);
            return q.getResultList();
        } catch (final NoResultException nre) {
            return new ArrayList<Facility>();
        }
    }
    
    public Facility findByStationNumber(String stationNumber) {
        try {
            final Query q = this.em.createNamedQuery("Facility.findByFacilityStation");
            q.setParameter("facilityStation", stationNumber);
            return (Facility) q.getSingleResult();
        } catch (final NoResultException nre) {
            return null;
        }
    }

    public List<Facility> findByStationNumbers(String stationNumbers) {
        try {
            final TypedQuery<Facility> q = this.em.createNamedQuery("Facility.findByFacilityStations", Facility.class);
            List<String> items = Arrays.asList(stationNumbers.split(","));
            q.setParameter("facilityStations", items);
            return q.getResultList();
        } catch (final NoResultException nre) {
            return null;
        }
    }
    
    public List<FacilityInfo> findFacilityInfo(String sortColumn, String sortDirection, final Integer maxRows, final Integer startRow){
        try{
            Query q = this.em.createNativeQuery(facilityInfoQuery + prepareOrderClause(sortColumn, sortDirection),"FacilityInfoRSM").setFirstResult(startRow);
            
            if( maxRows != -1 ) //set the max rows if we have it
            {
                q.setMaxResults(maxRows);
            } 
            
            //List<Object[]> rawResult = q.getResultList();
            List<FacilityInfo> rawResult = q.getResultList();
           
//            List<FacilityInfo> convertedResult = new ArrayList<>();
//            for(Object[] currentRow: rawResult){
//            
//                FacilityInfo data = new FacilityInfo((BigDecimal)currentRow[0], (String)currentRow[1], (String)currentRow[2], (String)currentRow[3], 
//                        (BigDecimal)currentRow[4], (BigDecimal)currentRow[5], (String)currentRow[6], (String)currentRow[7], (String)currentRow[8], (String)currentRow[9], 
//                        (String)currentRow[10], (String)currentRow[11], (String)currentRow[12], (String)currentRow[13], (String)currentRow[14], (String)currentRow[15]);
//                convertedResult.add(data);
//            }
//            return convertedResult;
            return rawResult;
        } catch (final NoResultException | IndexOutOfBoundsException ex) {
            return null;
        }
    }
    
    public FacilityInfo findSpecificFacilityInfo(String stationNumber){
        try{
                    
            Query q = this.em.createNativeQuery(facilityInfoQuery + whereClause,"FacilityInfoRSM");
            q.setParameter("stationNumber", stationNumber);
            
            //List<Object[]> rawResult = q.getResultList();
            List<FacilityInfo> rawResult = q.getResultList();
           
            //there should only ever be one result since station numbers should be unique
//            Object[] currentRow = rawResult.get(0);
//            FacilityInfo data = new FacilityInfo((BigDecimal)currentRow[0], (String)currentRow[1], (String)currentRow[2], (String)currentRow[3], 
//                    (BigDecimal)currentRow[4], (BigDecimal)currentRow[5], (String)currentRow[6], (String)currentRow[7], (String)currentRow[8], (String)currentRow[9], 
//                    (String)currentRow[10], (String)currentRow[11], (String)currentRow[12], (String)currentRow[13], (String)currentRow[14], (String)currentRow[15]);
            FacilityInfo data =  rawResult.get(0);
            return data;
        } catch (final NoResultException | IndexOutOfBoundsException ex) {
            return null;
        }
    }
    
    public List<Facility> findParentLevel() {
        try {
            return this.em.createNamedQuery("Facility.findParentLevel", Facility.class).getResultList();
        } catch (final NoResultException nre) {
            return null;
        }
    }
    
    /**
     * Finds the list of available parents, excluding the current.
     * @param stationNumber
     * @return 
     */
    public List<Facility> findParentLevel(String stationNumber) {
        try {
            TypedQuery<Facility> q = this.em.createNamedQuery("Facility.findParentLevelExcept", Facility.class);
            q.setParameter("stationNumber", stationNumber);
            
            return q.getResultList();
            
        } catch (final NoResultException nre) {
            return null;
        }
    }
    
    private String prepareOrderClause(String sortColumn, String sortDirection){
        //String tablePrefix = sortColumn == "VISN_NUMBER" ? "v." : "f.";
        String tablePrefix = sortColumn.equals( "VISN_NUMBER") ? "v." : "f.";
        String queryChunk = "ORDER BY " + tablePrefix + sortColumn + " ";
        
        if(sortDirection.equals("asc")){
            queryChunk = queryChunk + "ASC";
        } else {
            queryChunk = queryChunk + "DESC";
        }
        
        return queryChunk;
    }
    
     @Transactional
    public Facility update(final Facility fac)
            throws Exception {
        try {
            this.em.merge(fac);
        } catch (final Exception ex) {
            throw ex;
        }
        return fac;
    }
    
    @Transactional
    public Facility create(final Facility fac)
            throws Exception {
        try {
            this.em.persist(fac);
        } catch (final Exception ex) {
            throw ex;
        }
        return fac;
    }
    
}