package gov.va.nvap.web.facility;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.dao.FacilityDAO;
import gov.va.nvap.web.dao.VisnDAO;
import gov.va.nvap.web.visn.Visn;
import java.io.IOException;
//import java.io.StringWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;


/**
 *
 * @author Raul Alfaro
 */
public class EditFacility extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
        private static final long serialVersionUID = 1L;

	@Override
	protected void unspecified(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		final HttpSession session = request.getSession(false);

		Map<String, Object> finishedMap = encodeToMap(getResults(request));

		// Add available parents
		String selectedStation = request.getParameter("stationNumber");
		List<Facility> parentFacilities;

		if (!NullChecker.isNullOrEmpty(selectedStation)) {
			parentFacilities = getFacilityDAO().findParentLevel(selectedStation);
		} else {
			parentFacilities = getFacilityDAO().findParentLevel();
		}

		finishedMap.put("parentLevel", parentFacilities);
		session.setAttribute("results", finishedMap); // facilityInfoToJSON(finishedMap) );
                //request.setAttribute("stationNumber", selectedStation);
                
                this.forward(request, response, "show");
	}

	public Map<String, Object> encodeToMap(FacilityInfo rawResult) {
		Map<String, Object> endResult = new HashMap<String, Object>();
		List<Visn> visns = getVisnDAO().findAll();

		if (rawResult != null) {
			endResult.put("stationNumber", rawResult.getFacilityStation());
			endResult.put("facilityName", rawResult.getFacilityName());
			endResult.put("facilityDNS", rawResult.getDns());
			endResult.put("address", rawResult.getAddress());
			endResult.put("city", rawResult.getCity());
			endResult.put("state", rawResult.getState());
			endResult.put("zipCode", rawResult.getPostalCode());
			endResult.put("phoneNumber", rawResult.getPhone());
			endResult.put("visnId", rawResult.getVisnId());
			endResult.put("parent", (rawResult.getParentStationNumber() != null)
					? rawResult.getParentName() + " (" + rawResult.getParentStationNumber() + ")" : "None");
			endResult.put("parentStation",
					(rawResult.getParentStationNumber() != null) ? rawResult.getParentStationNumber() : "None");
			endResult.put("children", (rawResult.getChildren() != null) ? rawResult.getChildren() : "None");
			endResult.put("allowed", (rawResult.isAllowed()) ? "Yes" : "No");
		}

		endResult.put("visnList", visns);

		return endResult;
	}


        // Note: Removed the call to this method: returned a String to jsp page via session object, which can't be interpreted by JSTL tags.
        /*
    public String facilityInfoToJSON(Map<String, Object> hm) {
        String returnValue = "";
        
        if (hm != null) {
		        List<Visn> visnList = (List<Visn>) hm.get("visnList");
		        List<Facility> parentFacilities =  (List<Facility>) hm.get("parentLevel");
		        
		        Integer totalCount = new Integer(parentFacilities.size());
		        
		        try {
					StringWriter json = new StringWriter();
					JSONObject obj = new JSONObject();
					
					obj.put("stationNumber",hm.get("stationNumber"));
					obj.put("facilityName",hm.get("facilityName"));
					obj.put("facilityDNS",hm.get("facilityDNS"));
					obj.put("address",hm.get("address"));
					obj.put("city",hm.get("city"));
					obj.put("state",hm.get("state"));
					obj.put("zipCode",hm.get("zipCode"));
					obj.put("phoneNumber",hm.get("phoneNumber"));
					obj.put("visnId",hm.get("visnId"));
					obj.put("parent",hm.get("parent"));
					obj.put("parentStation",hm.get("parentStation"));
					obj.put("children",hm.get("children"));
					obj.put("allowed",hm.get("allowed"));

					obj.put("parentLevel",facilityToJSON(parentFacilities));
					obj.put("visn",visnToJSON(visnList));
					
					obj.put("recordsTotal", totalCount);
					obj.put("recordsFiltered", totalCount);
					obj.write(json);
			
			            returnValue = json.toString();
			            
			            //System.out.println(returnValue);
		        } catch (JSONException ex) {
//		            Logger.getLogger(ViewFacility.class.getName()).log(Level.SEVERE, null, ex);
		        }
    	}
        return returnValue;
    }
*/
/*
    public JSONArray visnToJSON(List<Visn> visnList) {
    	JSONArray dataArray = new JSONArray();
    	if( visnList != null){
	    	for(Visn visn : visnList){
	    		try {
	    			JSONObject dataItem = new JSONObject();
	    			dataItem.put("visn", (visn.getId() != null) ? visn.getId().toString() : "None");
	    			dataItem.put("VisnNumber", (visn.getVisnNumber() != null) ? visn.getId().toString() :"None");
	    			dataItem.put("VisnName", (visn.getVisnName() != null) ? visn.getId().toString() :"None");
	    			dataArray.put(dataItem);
				} catch (JSONException e) {
	
				}
	    	}
    	}
    	return dataArray;
    }
*/    
	public JSONArray facilityToJSON(List<Facility> parentFacilities) {

			JSONArray dataArray = new JSONArray();
			if (parentFacilities != null) {
				for (Facility current : parentFacilities) {
					try {				
						JSONObject dataItem = new JSONObject();
						dataItem.put("FacilityStation",current.getFacilityStation());
						dataItem.put("FacilityId",current.getFacilityId().toString());
						dataItem.put("FacilityDns",current.getFacilityDns());
						dataItem.put("FacilityName",current.getFacilityName());
						dataItem.put("VisnId",current.getVisnId().toString());
						dataItem.put("Address",current.getAddress());
						dataItem.put("City",current.getCity());
						dataItem.put("State",current.getState());
						dataItem.put("PostalCode",current.getPostalCode());
						dataItem.put("Phone",current.getPhone());
						dataItem.put("Parent",(current.getParent() != null) ? current.getParent() + " (" + current.getParent() + ")" : "None");
						dataItem.put("isAllowed",(current.isAllowed()) ? "Yes" : "No");
						dataItem.put("getActive",(current.getActive() != null) ? current.getActive() : "True");
						dataArray.put(dataItem);
					} catch (JSONException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

			}

		return dataArray;
	}
	
	public FacilityInfo getResults(HttpServletRequest request) {

		final String requestedStationNumber = request.getParameter("stationNumber");
		FacilityDAO facilityDao = getFacilityDAO();

		return facilityDao.findSpecificFacilityInfo(requestedStationNumber);
	}

	public void saveEdit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		Facility changedFacility = this.getFacilityDAO()
				.findByStationNumber(request.getParameter("originalStationNumber"));

                final HashMap<String, String> formFieldsBean = new HashMap<String, String>();
                String stationNumber = request.getParameter("stationNumber");
                formFieldsBean.put("stationNumber", stationNumber);
                String facilityName = request.getParameter("facilityName");
                formFieldsBean.put("facilityName", facilityName);
                String facilityDNS = request.getParameter("facilityDNS");
                formFieldsBean.put("facilityDNS", facilityDNS);
                String address = request.getParameter("address");
                formFieldsBean.put("address", address);
                String city = request.getParameter("city");
                formFieldsBean.put("city", city);
                String state = request.getParameter("state");
                formFieldsBean.put("state", state);
                String zipCode = request.getParameter("zipCode");
                formFieldsBean.put("zipCode", zipCode);
                String phoneNumber = request.getParameter("phoneNumber");
                formFieldsBean.put("phoneNumber", phoneNumber);
//                String visn = request.getParameter("visn");
//                formFieldsBean.put("visn", visn);
//                String parent = request.getParameter("parent");
//                formFieldsBean.put("parent", parent);
//                String allowed = request.getParameter("allowed");
//                formFieldsBean.put("allowed", allowed);
                boolean isValid = this.validate(formFieldsBean, request, response, "saveEdit");
                if (isValid) {
                    if (changedFacility != null) {
                        Facility facToSave = prepareForSave(changedFacility, request);
			this.getFacilityDAO().update(facToSave);

			// set the message to show on the list page
			request.setAttribute("facilityMessage", "Facility updated successfully.");
                    } else {
                            Facility facToCreate = prepareForSave(new Facility(), request);
                            this.getFacilityDAO().create(facToCreate);

                            // set the message to show on the list page
                            request.setAttribute("facilityMessage", "Facility created successfully.");
                    }
                    this.forward(request, response, "viewList");                    
                } else {
                    this.forward(request, response, "validate");
                }  
	}

	public Facility prepareForSave(Facility needsUpdating, HttpServletRequest request) {
                String stationNumber = request.getParameter("stationNumber");
		needsUpdating.setFacilityStation(stationNumber);
                
                String facilityName = request.getParameter("facilityName");
		needsUpdating.setFacilityName(facilityName);
                
                String facilityDNS = request.getParameter("facilityDNS");
		needsUpdating.setFacilityDns(facilityDNS);
                
                String address = request.getParameter("address");
		needsUpdating.setAddress(address);
                
                String city = request.getParameter("city");
		needsUpdating.setCity(city);
                
                String state = request.getParameter("state");
		needsUpdating.setState(state);
                
                String zipCode = request.getParameter("zipCode");
		needsUpdating.setPostalCode(zipCode);
                
                String phoneNumber = request.getParameter("phoneNumber");
		needsUpdating.setPhone(phoneNumber);
                
                String visn = request.getParameter("visn");
		needsUpdating.setVisnId(Long.parseLong(visn));
                
                String parent = request.getParameter("parent");
		needsUpdating.setParent(parent);
               
                String allowed = request.getParameter("allowed");
                boolean isAllowed = (allowed != null) ? true : false;
		needsUpdating.setAllowed(isAllowed);

		return needsUpdating;
	}

	public void inactivateFacility(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException, Exception {

		String stationNumber = request.getParameter("stationNumber");

		if (stationNumber != null) {
			Facility disable = this.getFacilityDAO().findByStationNumber(stationNumber);

			disable.setParent(null);

			// Set to false and update
			disable.setActive("False");
			this.getFacilityDAO().update(disable);
		}

		// set the message to show on the list page
		request.setAttribute("message", "Facility Inactivated Successfully.");

		this.forward(request, response, "viewList");
	}

	public void reactivateFacility(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException, Exception {
		String stationNumber = request.getParameter("stationNumber");

		if (stationNumber != null) {
			Facility activate = this.getFacilityDAO().findByStationNumber(stationNumber);

			// Set to false and update
			activate.setActive("True");
			this.getFacilityDAO().update(activate);
		}

		// set the message to show on the list page
		request.setAttribute("message", "Facility Reactivated Successfully.");

		this.forward(request, response, "viewList");
	}

	private FacilityDAO getFacilityDAO() {
		return this.getBean("FacilityDAO", FacilityDAO.class);
	}

	private VisnDAO getVisnDAO() {
		return this.getBean("VisnDAO", VisnDAO.class);
	}
}
