/**
 *
 */
package gov.va.nvap.web.report;


import gov.va.nvap.web.helper.report.DqTransmissionLogReportHelper;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.service.audit.AuditException;
import gov.va.nvap.service.dataquality.DqTransmissionLog;
import gov.va.nvap.service.dataquality.DqTransmissionLogManager;
import gov.va.nvap.service.dataquality.DqTransmissionLogQueryRequest;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.patient.ExcelGeneratorThread;
import gov.va.nvap.web.user.UserHelper;
import gov.va.nvap.web.util.Constants;
import gov.va.nvap.web.util.xls.CsvExporter;
import gov.va.nvap.web.util.xls.ExcelExporter;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.ss.usermodel.Workbook;

import gov.va.nvap.web.dao.UserDocumentDAO;
import org.apache.commons.lang.StringUtils;
import org.owasp.esapi.ESAPI;
import org.owasp.esapi.filters.ESAPIFilter;


/**
 *
 * 
 */
public class DqTransmissionLogReport extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
    private static final long serialVersionUID = 1L;
    
    static private final Log LOG = LogFactory.getLog(DqTransmissionLogReport.class);
    
    private final String REPORT_TITLE = "Data Quality Upload Transmission Log Report";
    
    private DqTransmissionLogManager dqTransmissionLogManager;   

    private DqTransmissionLogReportHelper dqtlReportHelper;
    
    public DqTransmissionLogReport() {
        super();
        this.dqtlReportHelper = new DqTransmissionLogReportHelper();
    }
    
    @EJB(beanInterface = DqTransmissionLogManager.class, mappedName = "DqTransmissionLogManager")
    public void setDqTransmissionLogManager(final DqTransmissionLogManager dqTransmissionLogManager) {
        this.dqTransmissionLogManager = dqTransmissionLogManager;
    }

    @Override
    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        final HttpSession session = request.getSession(false);

        //this is the search form
	request.setAttribute("pouFlag", Constants.PURPOSE_OF_USE_FLAG);
        ReportHelper.setDefaultSearchDates(session);

        this.forward(request, response, "searchForm");
    }
    
    public void doSearch(final HttpServletRequest request, final HttpServletResponse response) throws AuditException, IOException, ServletException {
        LOG.debug("dqTransmissionLogManager.doSearch() called");
        final HttpSession session = request.getSession(false);
        try { 
            
            // get the input parameters for the data query request and place into new request object            
            final Date runStartDate = ReportHelper.getStartDate(getParamWithoutHPP(request,"startDate"));
            final Date runEndDate = ReportHelper.getEndDate(getParamWithoutHPP(request,"endDate"));
            final String filename = getParamWithoutHPP(request, "filename"); // Fix to avert Http Parameter Pollution
            final String eventType = getParamWithoutHPP(request,"eventType");
            final String status = getParamWithoutHPP(request,"status");
   
            
            
            // set the input into the query request object
            DqTransmissionLogQueryRequest logQueryRequest = new DqTransmissionLogQueryRequest();
            logQueryRequest.setRunStartDate(runStartDate);
            logQueryRequest.setRunEndDate(runEndDate);            
            if (!NullChecker.isNullOrEmpty(filename) ) {
                logQueryRequest.setFilename(filename);
            }
            if (!NullChecker.isNullOrEmpty(eventType)) {
                logQueryRequest.setEventType(eventType);
            }
            if (!NullChecker.isNullOrEmpty(status)) {
                logQueryRequest.setStatus(status);
            }            
           
            // make the query and store the response
            String results = this.dqTransmissionLogManager.getDqTransmissionLogData(logQueryRequest);
            if(!NullChecker.isNullOrEmpty(results)) {
                // reformat the JSON string results for DataTables JavaScript library to use as input:
                // Transform from JSON String to Object
                DqTransmissionLogReportHelper.QueryResponseObj respObj = this.dqtlReportHelper.transformRespJsonStrToQueryResponseObj(results);
                // Save these raw Object results in the session for other methods
                session.setAttribute("DqTransmissionLogReportResults", respObj.getData());
                // Transform from Object to Object, and Object to JSON String for DataTables/AJAX input   
                String transformedResultsJsonStr = this.dqtlReportHelper.transformJsonRespObjToDataTablesJsonStr(respObj);                
                // set the results into the session
                session.setAttribute("results", transformedResultsJsonStr);
            } else {
                LOG.debug("dqTransmissionLogManager.getDqTransmissionLogData() empty results:" + results);
                session.setAttribute("results", "");
            }
            
            LOG.debug("dqTransmissionLogManager.getDqTransmissionLogData() returned results that were not null or empty.");
            // forward to the display jsp to output as json String for ajax response
            this.forward(request, response, "searchResultsJSON");
        } catch (Exception e) {
            //LOG.error( "DqTransmissionLogReport doSearch threw Exception:", e );
            throw new ServletException("DqTransmissionLogReport doSearch threw Exception:", e);
        }        
    }
    
     /**
      * Method to get Parameters from Request without HTTP Parameter Pollution
      * @param  request   -- The http request to get the attributes from
      * @param  paramName -- The name of the http attribute to be retrieved
      * @return the single value of the request Param
      * @throws ServletException
      * 
      */
      
	private String getParamWithoutHPP(HttpServletRequest request, String paramName) throws ServletException
	{
		String[] paramValues = request.getParameterValues(paramName);
		if(paramValues ==null || paramValues.length==0)
		{
			return StringUtils.EMPTY;
		}
		else if(paramValues.length > 1)
		{
			throw new ServletException(" Multiple request values, HPP attack averted ");
		}
		return paramValues[0];
	}
    
    private ReportHelper getReportHelper() {
        return this.getBean("reportHelper", ReportHelper.class);
    }
    
    public void exportToExcel(final HttpServletRequest request,
        final HttpServletResponse response) throws ServletException,
        IOException {
        
        // get the request search parameter values from request
        final Date startDateStr = ReportHelper.getStartDate(request.getParameter("startDate"));
        final Date endDateStr = ReportHelper.getEndDate(request.getParameter("endDate"));
        final String eventType = request.getParameter("eventType");
        final String status = request.getParameter("status");
        final String filename = request.getParameter("filename");
        // Generate filters.
        final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
        final LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();
        // set the request parameters
        //filters.put("User ID", request.getParameter("userId"));
        filters.put("Start Date", this.getReportHelper().getFormattedDate(startDateStr));
        filters.put("End Date", this.getReportHelper().getFormattedDate(endDateStr));
        filters.put("Event Type", eventType);
        filters.put("Status", status);
        filters.put("File Name", filename);
        ExcelExporter.populateFilterMapForExport(request, filters, filterMap, null);

        // Create a map of key which is based on the result data key and the
        // heading value
        // The heading is used to create the column headers and the key is used
        // to pull the data from the results
        // set the document column headers for the output
        final Map<String, String> inboundDocumentMap = new LinkedHashMap<String, String>();
        inboundDocumentMap.put("dqTransmissionId", "DQ Transmission Id");
        inboundDocumentMap.put("batchId", "Batch Id");
        inboundDocumentMap.put("eventType", "Event Type");
        inboundDocumentMap.put("startTimeFormatted", "Start Time (CT)");
        inboundDocumentMap.put("endTimeFormatted", "End Time (CT)");
        inboundDocumentMap.put("hostName", "Host Name");
        inboundDocumentMap.put("fileName", "File Name");
        inboundDocumentMap.put("fileType", "File Type");
        inboundDocumentMap.put("fileSize", "File Size");
        inboundDocumentMap.put("destination", "Destination");
        inboundDocumentMap.put("status", "Status");
        inboundDocumentMap.put("localDocId", "Local Doc Id");
        inboundDocumentMap.put("remoteDocId", "Remote Doc Id");
        inboundDocumentMap.put("error", "Error");

        // test for scheduled export output or direct download output based on the size of existing query results
        int minimumThreshold = Integer.parseInt((String) this.getServletContext().getAttribute("scheduledExportMin"));
        int total = Integer.parseInt(request.getParameter("totalRows"));

        // Start a scheduled export with its own thread if greater than the minimum threshold.
        // It will use the query parameters as input to query the data source all over again, 
        // and then output the document in the Scheduled Export page instead of the current page.
        if (total >= minimumThreshold) {
            ExcelGeneratorThread exGenThread = new ExcelGeneratorThread("exGenThread");
            exGenThread.setFilterMap(filterMap);
            exGenThread.setReportMap(inboundDocumentMap);

            // Set names for Excel generator thread
            exGenThread.setTitle(REPORT_TITLE);

            exGenThread.setExcelExporter(this.getExcelExporter());
            exGenThread.setDqTransmissionLogManager(this.dqTransmissionLogManager);
            exGenThread.setReportHelper(getReportHelper());
            exGenThread.setDqtlReportHelper(this.dqtlReportHelper);
            exGenThread.setUserId(UserHelper.getUserName(request));
            exGenThread.setUserDocumentDao(getUserDocumentDAO());

            // Set search attributes for getting log results
            exGenThread.setAttributes(this.dqtlReportHelper.mapSessionAttributes(request));

            exGenThread.run();
        }
        // Otherwise generate and download the export directly
        else {
            // Create workbook            
            List<DqTransmissionLog> data = 
                (List<DqTransmissionLog>)request.getSession().getAttribute("DqTransmissionLogReportResults");
            
           
            List<Map<String, Object>> results = this.dqtlReportHelper.transformDataIntoExcelFormat(data);

            final Workbook wb = this.getExcelExporter().exportToExcel(REPORT_TITLE, REPORT_TITLE, inboundDocumentMap, results, filterMap, null);

            // Write Excel workbook to Stream
            this.getExcelExporter().writeExcelToStream(REPORT_TITLE.replace(" ", "_"), wb, response);
        }
    }    

    private UserDocumentDAO getUserDocumentDAO() {
        return this.getBean("UserDocumentDAO", UserDocumentDAO.class);
    }
    
    public ExcelExporter getExcelExporter() {
            final ExcelExporter excelExporter = this.getBean("excelExporter",
                            ExcelExporter.class);
            return excelExporter;
    }
    
    public void exportToCsv(final HttpServletRequest request,
        final HttpServletResponse response) throws ServletException, IOException {

        // set the document column headers for the output
        Map<String, String> csvReportMap = new LinkedHashMap<String, String>();
        csvReportMap.put("DQ Transmission Id", "dqTransmissionId");
        csvReportMap.put("Batch Id", "batchId");
        csvReportMap.put("Event Type", "eventType");
        csvReportMap.put("Start Time (CT)", "startTimeFormatted");
        csvReportMap.put("End Time (CT)", "endTimeFormatted");
        csvReportMap.put("Host Name", "hostName");
        csvReportMap.put("File Name", "fileName");
        csvReportMap.put("File Type", "fileType");
        csvReportMap.put("File Size", "fileSize");
        csvReportMap.put("Destination", "destination");
        csvReportMap.put("Status", "status");
        csvReportMap.put("Local Doc Id", "localDocId");
        csvReportMap.put("Remote Doc Id", "remoteDocId");
        csvReportMap.put("Error", "error");      
        
        // test for scheduled export output or direct download output based on the size of existing query results
        int minimumThreshold = Integer.parseInt((String) this.getServletContext().getAttribute("scheduledExportMin"));
        int total = Integer.parseInt(request.getParameter("totalRows"));

        // Start a scheduled export with its own thread if greater than the minimum threshold.
        // It will use the query parameters as input to query the data source all over again, 
        // and then output the document in the Scheduled Export page instead of the current page
        if (total >= minimumThreshold) {
            ExcelGeneratorThread csvGenThread = new ExcelGeneratorThread("csvGenThread");
            csvGenThread.setDocumentType("csv");
            csvGenThread.setReportMap(csvReportMap);

            csvGenThread.setTitle(REPORT_TITLE);
            csvGenThread.setUserId(UserHelper.getUserName(request));
            csvGenThread.setDqTransmissionLogManager(this.dqTransmissionLogManager);
            csvGenThread.setReportHelper(getReportHelper());
            csvGenThread.setDqtlReportHelper(this.dqtlReportHelper);
            csvGenThread.setUserDocumentDao(getUserDocumentDAO());
            csvGenThread.setCsvExport(new CsvExporter());

            // Set search attributes for getting audit results
            csvGenThread.setAttributes(this.dqtlReportHelper.mapSessionAttributes(request));

            csvGenThread.start();
        }
        // Otherwise generate and download the export directly
        else {
            List<DqTransmissionLog> data = 
                (List<DqTransmissionLog>)request.getSession().getAttribute("DqTransmissionLogReportResults");
            
           
            List<Map<String, Object>> results = this.dqtlReportHelper.transformDataIntoExcelFormat(data);

            CsvExporter csvExporter = new CsvExporter();
            csvExporter.exportToCSV(response, REPORT_TITLE.replace(" ", "_"), results, (LinkedHashMap<String, String>) csvReportMap);
        }
    }
  
    /**
     * Get the value of dqtlReportHelper
     *
     * @return the value of dqtlReportHelper
     */
    public DqTransmissionLogReportHelper getDqtlReportHelper() {
        return dqtlReportHelper;
    }

    /**
     * Set the value of dqtlReportHelper
     *
     * @param dqtlReportHelper new value of dqtlReportHelper
     */
    public void setDqtlReportHelper(DqTransmissionLogReportHelper dqtlReportHelper) {
        this.dqtlReportHelper = dqtlReportHelper;
    }
    
}
