package gov.va.nvap.web.util.date;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Utility to parse dates.
 * 
 * @author Asha Amritraj
 * 
 */
public class DateUtil {

    public static Date addTime(final Date date, final int field,
                    final int amount) {
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(field, amount);
        final Date newDate = cal.getTime();
        return newDate;
    }

    public static Date setTime(final Date date, final int field,
                    final int amount) {
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        // use set instead of add here, to avoid increasing time:
        cal.set(field, amount);
        final Date newDate = cal.getTime();
        return newDate;
    }

    public static Date setTimeOnTheHour(final Date date, final int hour) {
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        // use set instead of add here, to avoid increasing time:
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        final Date newDate = cal.getTime();
        return newDate;
    }
    
    public static Date addDays(Date date, int days) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.DATE, days); //minus number would decrement the days
        return cal.getTime();
    }
    
    public static Date addMonths(Date date, int numberOfMonths) {
        Calendar cal = Calendar.getInstance(); 
        cal.setTime(date);
        cal.add(Calendar.MONTH, numberOfMonths);
        return cal.getTime();
    }
    
    public static Date setDayOfMonth(Date date, int dayOfMonth) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, dayOfMonth); 
        return cal.getTime();
    }
    
    public static int getDayOfWeek(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.DAY_OF_WEEK);
    }
    
    public static int getDayOfMonth(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.DAY_OF_MONTH);
    }
    
    public static int getLastDayOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.getActualMaximum(Calendar.DAY_OF_MONTH);
    }
    
    /**
     * Parse date MM/dd/yyyy.
     */
    public static Date parseDateMMddyyyy(final String dateString) {
        final SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
        try {
                Date returnDate = formatter.parse(dateString);
                return returnDate;
        } catch (final ParseException t) {
                throw new RuntimeException(t);
        }
    }

    /**
     * Time zero! In the beginning...
     */
    public static Date voidTime(final Date date) {
        final Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }
    
    public static int getAge(Date dateOfBirth) {
        Calendar today = Calendar.getInstance();
        Calendar birthDate = Calendar.getInstance();
        birthDate.setTime(dateOfBirth);
        if (birthDate.after(today)) {
            throw new IllegalArgumentException("You don't exist yet");
        }
        int todayYear = today.get(Calendar.YEAR);
        int birthDateYear = birthDate.get(Calendar.YEAR);
        int todayDayOfYear = today.get(Calendar.DAY_OF_YEAR);
        int birthDateDayOfYear = birthDate.get(Calendar.DAY_OF_YEAR);
        int todayMonth = today.get(Calendar.MONTH);
        int birthDateMonth = birthDate.get(Calendar.MONTH);
        int todayDayOfMonth = today.get(Calendar.DAY_OF_MONTH);
        int birthDateDayOfMonth = birthDate.get(Calendar.DAY_OF_MONTH);
        int age = todayYear - birthDateYear;

        // If birth date is greater than todays date (after 2 days adjustment of leap year) then decrement age one year
        if ((birthDateDayOfYear - todayDayOfYear > 3) || (birthDateMonth > todayMonth)){
            age--;

        // If birth date and todays date are of same month and birth day of month is greater than todays day of month then decrement age
        } else if ((birthDateMonth == todayMonth) && (birthDateDayOfMonth > todayDayOfMonth)){
            age--;
        }
        return age;
    }
}
