package gov.va.nvap.web.util.status;

import gov.va.med.nhin.adapter.audit.ActionType;
import gov.va.med.nhin.adapter.audit.ActionValuesType;
import gov.va.med.nhin.adapter.audit.GetAudits;
import gov.va.med.nhin.adapter.audit.StringValuesType;
import gov.va.nvap.audit.BulkDownloadDocs;
import gov.va.nvap.common.exception.StackTraceUtil;
import gov.va.nvap.service.adapter.audit.AdapterAuditManager;
import gov.va.nvap.service.adapter.audit.DirectAuditManager;
import gov.va.nvap.service.adapter.audit.DirectServiceEndpoint;
import gov.va.nvap.service.pdq.PatientDemographicsQuery;
import gov.va.nvap.service.pdq.PatientMatchQuery;
import gov.va.nvap.service.pdq.PatientProfile;
import gov.va.nvap.service.pdq.PdqService;
import gov.va.nvap.svc.consentmgmt.stub.FacilityResolver;
import gov.va.nvap.web.patient.PatientSearch;
import gov.va.nvap.web.report.ReportDataProcessor;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Month;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.identityconnectors.common.security.GuardedString;
import org.json.JSONException;
import org.json.JSONObject;
import weblogic.security.internal.SerializedSystemIni;
import weblogic.security.internal.encryption.ClearOrEncryptedService;
import weblogic.security.internal.encryption.EncryptionService;

/**
 *
 * @author Johann P. Sonnenberg
 */

public class StatusChecker extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet{
    private static final long serialVersionUID = 1L;
    
    @Override
    public void unspecified(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        //default page
        this.forward(request, response, "show");
    }
    
    public void checkExchangeStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
            final GetAudits getAuditsRequest = new GetAudits();

            final StringValuesType patientSSNs = new StringValuesType();
            patientSSNs.getValue().add("000000000");
            patientSSNs.setNotIn(false);
            getAuditsRequest.setPatientSSNs(patientSSNs);

            final ActionValuesType actionsType = new ActionValuesType();
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
            getAuditsRequest.setActions(actionsType);

            this.adapterAuditManager.getAudits(getAuditsRequest);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        } 
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("Exchange", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkMVIStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            PatientProfile testPatient = new PatientProfile();
            // java.util.Date was deprecated in JEE 8
            //testPatient.setDob(new Date("01/01/1970"));
            LocalDate birthDay = LocalDate.of(1970, Month.JANUARY, 01);
            Date birthDate = java.sql.Date.valueOf(birthDay);
            testPatient.setDob(birthDate);             
            testPatient.setFirstName("CHDRONE");
            testPatient.setLastName("CHDRONE");
            testPatient.setSsn("000000000");
            
            PatientMatchQuery pmq = new PatientMatchQuery();
            
            pmq.setPatientProfile(testPatient);
            
            this.pdqService.searchPatient(pmq);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("MVI", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkDirectStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            final GetAudits getAuditsRequest = new GetAudits();

            final StringValuesType patientSSNs = new StringValuesType();
            patientSSNs.getValue().add("000000000");
            patientSSNs.setNotIn(false);
            getAuditsRequest.setPatientSSNs(patientSSNs);

            final ActionValuesType actionsType = new ActionValuesType();
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
            getAuditsRequest.setActions(actionsType);
            
            this.directAuditManager.getAudits(getAuditsRequest);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("Direct", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkVistAStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try{
            PatientDemographicsQuery query = new PatientDemographicsQuery();
            query.setPatientId("1012581676V377802");
            
            pdqService.getPatientDemographics(query);
        } catch(Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("VistA", pass, error));

        this.forward(request, response, "resultsJSON");
    }
    
    public void checkESRStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
           this.facilityResolver.getPatientPreferredFacility("000000000");
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("ESR", pass, error));
        
        this.forward(request, response, "resultsJSON");
    }
    
    public void checkBulkDownloadStatus(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        Boolean pass = true;
        String error = "";
        
        //do the call to the service
        try {
            SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
            String today = formatter.format(new Date());
            ClearOrEncryptedService encSvc;
            String domainHome = new File("dummy").getAbsoluteFile().getParent();
            EncryptionService encryptSrv = SerializedSystemIni.getEncryptionService(domainHome);
            encSvc = new ClearOrEncryptedService(encryptSrv);
            
            
            GuardedString vapIdClear = ReportDataProcessor.toGuardedString(encSvc.decrypt(this.loadVAPId()));
            getbulkDownloadDocs().downloadRetrievedDocs(ReportDataProcessor.decryptToString(vapIdClear), today, today, "1", "0000000", "000000000", "TestPatient", "TestPatient");
        } catch (Exception e) {
            pass = false;
            error = StackTraceUtil.getStackTraceUtilAsString(e);
        }
        
        //package it up into JSON and send it back to the caller
        request.setAttribute("results", translateToJSON("BulkDownload", pass, error));
        
        this.forward(request, response, "resultsJSON");
    }
    
    private String translateToJSON(String serviceChecked, Boolean pass, String error) {
        String jsonString = "";
        
        DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy h:mm:ss a");
        
        Calendar todayCal = Calendar.getInstance();
        String today = dateFormat.format(todayCal.getTime());
        
        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            
            obj.put("serviceChecked", serviceChecked);
            obj.put("pass", pass);
            obj.put("error", error);
            obj.put("dateTimeChecked", today);
            
            obj.write(json);

            jsonString = json.toString();
        } catch (JSONException ex) {
            Logger.getLogger(PatientSearch.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        return jsonString;
    }
    
    //used for calls to the Exchange Service
    private AdapterAuditManager adapterAuditManager;
    
    @EJB(beanInterface = AdapterAuditManager.class, mappedName = "AdapterAuditManager")
    
    public void setAdapterAuditManager(final AdapterAuditManager adapterAuditManager) {
        this.adapterAuditManager = adapterAuditManager;
    }
    
    //used for calls to MVI Service and VistA Service
    private PdqService pdqService;
    
    @EJB(beanInterface = PdqService.class, mappedName = "PdqService")
    
    public void setPdqService(final PdqService pdqService) {
        this.pdqService = pdqService;
    }
    
    //Direct Service
    private final DirectAuditManager directAuditManager = new DirectServiceEndpoint();
    
    
    //used for calls to the ESR Service
    private FacilityResolver facilityResolver;
    
    @EJB(beanInterface = FacilityResolver.class, mappedName = "FacilityResolver")
    
    public void setFacilityResolver(final FacilityResolver facilityResolver) {
        this.facilityResolver = facilityResolver;
    }
    
    //used to make calls to the bulk document download
    public BulkDownloadDocs getbulkDownloadDocs()
    {
    	String useMock =""; 
    	useMock = System.getProperty("mock.bulk.download")!=null?System.getProperty("mock.bulk.download"):"";
    	String mockClass="";    
        
    	 if(useMock.equalsIgnoreCase("true"))
         {
    	      mockClass = "bulkDownloadDocs-mock";     		 
    	 }    	   
    	 else
    	 {
    	      mockClass = "bulkDownloadDocs";    		
    	 }
         
        final BulkDownloadDocs bulkDownloadBean = getBean(mockClass, BulkDownloadDocs.class);
        
    	return bulkDownloadBean;    	
    }
    
    //used to get the VAP_ID from a properties file so we can use it to make the call to check on the bulk document download service
    private String loadVAPId() throws IOException
    {
        File file;
        
        if (System.getProperty("vap.bulkDownload.config") != null) {
            final String path = FilenameUtils.normalize(System.getProperty("vap.bulkDownload.config"));
            file = new File(path);
        } else {
            file = new File("config/vap/bulkDownload.properties");
        }

        InputStream stream = null;
        Properties props = new Properties();
        
        try {
            stream = new FileInputStream(file);
            props.load(stream);
        } finally {
            IOUtils.closeQuietly(stream);
        }
        
        return props.getProperty("vap.identifier");
    }
}