package gov.va.nvap.web.util.xls;

import gov.va.nvap.web.dao.UserDocumentScheduleAuditDAO;
import gov.va.nvap.web.dao.UserDocumentScheduleDAO;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.user.UserHelper;
import gov.va.nvap.web.user.document.*;
import gov.va.nvap.web.util.date.DateUtil;
import java.io.IOException;
import java.io.StringWriter;
import static java.lang.Math.toIntExact;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author Johann P. Sonnenberg
 */
public class ManageScheduledExports extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {

    private static final long serialVersionUID = 1L;
    
    private UserDocumentScheduleDAO getUserDocumentScheduleDAO() {
        return this.getBean("UserDocumentScheduleDAO", UserDocumentScheduleDAO.class);
    }
    
    private UserDocumentScheduleAuditDAO getUserDocumentScheduleAuditDAO() {
        return this.getBean("UserDocumentScheduleAuditDAO", UserDocumentScheduleAuditDAO.class);
    }
    
    private ReportHelper getReportHelper() {
        return this.getBean("reportHelper", ReportHelper.class);
    }
    
    @Override
    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        this.forward(request, response, "show");
    }
    
     public void getUserExports(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        //Get paramaters from Datables request
        final HttpSession session = request.getSession(false);
        final String maxRows = request.getParameter("length");
        final String startRow = request.getParameter("start");
        final String draw = request.getParameter("draw");
        final String sortBy = request.getParameter("order[0][column]");
        final String sortOrder = request.getParameter("order[0][dir]");

        //Get the exports with the parameters from Datables
        HashMap<String, Object> hm = this.getUserDocumentScheduleDAO().getUserDocumentScheduledForUser(UserHelper.getUserName(request), sortBy, sortOrder, Integer.parseInt(maxRows), Integer.parseInt(startRow));
        
        session.setAttribute("results", encodeToJSON(hm, draw));

        this.forward(request, response, "resultsJSON");
    }
    
    public void addUserScheduledExportAttempt(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        boolean success = false;
        String message = "";
        UserDocumentSchedule uds = new UserDocumentSchedule();
        
        //get parameters from the form
        final String reportType = request.getParameter("reportType");
        final String userDefinedReportName = request.getParameter("userDefinedReportName");
        final String format = request.getParameter("format");
        final String reportParameters = request.getParameter("scheduledExportParametersJSON");
        final String reportViewParameters = request.getParameter("scheduledExportViewParametersJSON");
        final String frequency = request.getParameter("frequency");
        final String endDateSelection = request.getParameter("endDateSelection");
        final String periodOfReporting = request.getParameter("periodOfReporting");
        
        Long weeklyRecurrenceWeek = null;
        Long weeklyRecurrenceDayOfWeek = null;
        Long monthlyRecurrenceMonth = null;
        String monthlyRecurrenceDay = "";
        
        Date endDate = null;
        
        if(frequency.equals("weekly")) {
            //load up the "weekly" properties
            weeklyRecurrenceWeek = Long.parseLong(request.getParameter("weeklyRecurrences"));
            weeklyRecurrenceDayOfWeek = Long.parseLong(request.getParameter("frequencyWeeklyDayOfWeek"));
            
            uds.setWeeklyRecurrenceDayOfWeek(weeklyRecurrenceDayOfWeek);
            uds.setWeeklyRecurrenceWeek(weeklyRecurrenceWeek);
        }
        
        if(frequency.equals("monthly")) {
            //load up the "monthly" properties
            monthlyRecurrenceMonth = Long.parseLong(request.getParameter("monthlyOccurrences"));
            monthlyRecurrenceDay = request.getParameter("monthlyOccurrenceInMonth");
            
            uds.setMonthlyRecurrenceMonth(monthlyRecurrenceMonth);
            uds.setMonthlyRecurrenceDay(monthlyRecurrenceDay);
        }
        
        //calculate end_date
        if(endDateSelection.equals("endAfter")) {
            final int endAfterOccurrences = Integer.parseInt(request.getParameter("endAfterOccurrences"));
            
            endDate = this.calculateEndDate(frequency, endAfterOccurrences, weeklyRecurrenceWeek, weeklyRecurrenceDayOfWeek, monthlyRecurrenceMonth, monthlyRecurrenceDay);
        }
        
        //end date has been entered
        if(endDateSelection.equals("endBy")) {
            endDate = DateUtil.parseDateMMddyyyy(request.getParameter("scheduledEndDate"));
            endDate = DateUtil.setTimeOnTheHour(endDate, 6);
        }
        
        //no end date 
        if(endDateSelection.equals("noEnd")) {
            endDate = null; //don't REALLY need this here but just in case some day we have to do something with a blank or null endDate
        }
        
        try {
            //load up the UserDocumentSchedule object with the remaining "common" properties
            uds.setReportType(reportType);
            uds.setUserDefinedReportName(userDefinedReportName);
            uds.setFormat(format);
            uds.setPeriodOfReporting(periodOfReporting);
            uds.setReportParameters(reportParameters);
            uds.setReportViewParameters(reportViewParameters);
            uds.setFrequency(frequency.toUpperCase());
            uds.setStartDate(DateUtil.voidTime(new Date()));
            uds.setCreationDate(new Date());
            uds.setLastUpdatedDate(new Date());
            uds.setUserId(UserHelper.getUserName(request));
            uds.setEndDate(endDate);
            uds.setPeriodOfReporting(periodOfReporting);
            
            //make the call to the database to add the user scheduled export
            this.getUserDocumentScheduleDAO().create(uds);
            
            success = true;
            message = "This report has been successfully set for advanced scheduling. To edit the advanced schedule options for this task, navigate to the \"Advanced Scheduling\" page, under the \"Welcome\" menu.";
        } catch(Exception e) {
            success = false;
            message = "An error occurred while attempting to complete your request for advanced scheduling of a report. Please try again.";
        }
        
        //set up the return json
        StringWriter json = new StringWriter();
        JSONObject obj = new JSONObject();
        
        try {    
            obj.put("success", success);
            obj.put("message", message);
            obj.write(json);
        } catch (JSONException ex) { 
            //do nothing
        }
        
        request.setAttribute("results", json.toString());
        
        this.forward(request, response, "resultsJSON");
    }
    
    public void editUserScheduledExport(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        final Long scheduleId = Long.parseLong(request.getParameter("id"));
        
        //get the user scheduled export to edit
        UserDocumentSchedule uds = new UserDocumentSchedule(); 
        
        uds = this.getUserDocumentScheduleDAO().getUserDocumentSchedule(scheduleId);
        
        request.setAttribute("userDocumentSchedule", uds);
    
        this.forward(request, response, "editUserScheduledExport");
    }
    
    public void editUserScheduledExportAttempt(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        //get parameters from the form
        final Long scheduleId = Long.parseLong(request.getParameter("scheduleId"));
        final String userDefinedReportName = request.getParameter("userDefinedReportName");
        final String format = request.getParameter("format");
        final String reportParameters = request.getParameter("scheduledExportParametersJSON");
        final String reportViewParameters = request.getParameter("scheduledExportViewParametersJSON");
        final String frequency = request.getParameter("frequency");
        final String endDateSelection = request.getParameter("endDateSelection");
        final String periodOfReporting = request.getParameter("periodOfReporting");
        
        Long weeklyRecurrenceWeek = null;
        Long weeklyRecurrenceDayOfWeek = null;
        Long monthlyRecurrenceMonth = null;
        String monthlyRecurrenceDay = "";
        
        Date endDate = null;
        
        //get our object we need to work with
        UserDocumentSchedule uds = this.getUserDocumentScheduleDAO().getUserDocumentSchedule(scheduleId);
        
        //figure out frequency and end date
        if(frequency.equals("weekly")) {
            //load up the "weekly" properties
            weeklyRecurrenceWeek = Long.parseLong(request.getParameter("weeklyRecurrences"));
            weeklyRecurrenceDayOfWeek = Long.parseLong(request.getParameter("frequencyWeeklyDayOfWeek"));
            
            uds.setWeeklyRecurrenceDayOfWeek(weeklyRecurrenceDayOfWeek);
            uds.setWeeklyRecurrenceWeek(weeklyRecurrenceWeek);
        }
        
        if(frequency.equals("monthly")) {
            //load up the "monthly" properties
            monthlyRecurrenceMonth = Long.parseLong(request.getParameter("monthlyOccurrences"));
            monthlyRecurrenceDay = request.getParameter("monthlyOccurrenceInMonth");
            
            uds.setMonthlyRecurrenceMonth(monthlyRecurrenceMonth);
            uds.setMonthlyRecurrenceDay(monthlyRecurrenceDay);
        }
        
        //calculate end_date
        if(endDateSelection.equals("endAfter")) {
            final int endAfterOccurrences = Integer.parseInt(request.getParameter("endAfterOccurrences"));
            
            endDate = this.calculateEndDate(frequency, endAfterOccurrences, weeklyRecurrenceWeek, weeklyRecurrenceDayOfWeek, monthlyRecurrenceMonth, monthlyRecurrenceDay);
        }
        
        //end date has been entered
        if(endDateSelection.equals("endBy")) {
            endDate = DateUtil.parseDateMMddyyyy(request.getParameter("scheduledEndDate"));
            endDate = DateUtil.setTimeOnTheHour(endDate, 6);
        }
        
        //no end date 
        if(endDateSelection.equals("noEnd")) {
            endDate = null; 
        }
        
        try {
            //load it up with the common properties
            uds.setUserDefinedReportName(userDefinedReportName);
            uds.setFormat(format);
            uds.setPeriodOfReporting(periodOfReporting);
            uds.setReportParameters(reportParameters);
            uds.setReportViewParameters(reportViewParameters);
            uds.setFrequency(frequency.toUpperCase());
            uds.setLastUpdatedDate(new Date());
            uds.setEndDate(endDate);
            uds.setPeriodOfReporting(periodOfReporting);
            
            //save it
            this.getUserDocumentScheduleDAO().update(uds);
            
        } catch(Exception e) {
            //fail silently for now...
        }
        
        request.setAttribute("advancedSchedulingMessage", "The advanced scheduling task " + uds.getUserDefinedReportName() + " save was successful.");
        
        this.forward(request, response, "show");
    }
    
    public void removeUserScheduledExportAttempt(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        final String removalIds = request.getParameter("ids");
        
        //send to the DAO to do the delete 
        this.getUserDocumentScheduleDAO().deleteByIds(UserHelper.getUserName(request), removalIds);
        
        //set up the return json
        StringWriter json = new StringWriter();
        JSONObject obj = new JSONObject();
        
        try {
            obj.put("success", "true");
            obj.put("message", "Advanced scheduling task(s) successfully removed.");
            obj.write(json);
        } catch (JSONException ex) { 
            //do nothing
        }
        
        request.setAttribute("results", json.toString());
        
        this.forward(request, response, "resultsJSON");
    }
    
    public void getUserScheduledExportLog(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        final Long scheduleId = Long.parseLong(request.getParameter("id"));
        
        UserDocumentSchedule uds = this.getUserDocumentScheduleDAO().getUserDocumentSchedule(scheduleId);
        
        List<UserDocumentScheduleAudit> logs = (List<UserDocumentScheduleAudit>) this.getUserDocumentScheduleAuditDAO().getUserDocumentScheduledAudits(scheduleId).get("results");
        
        request.setAttribute("userDocumentSchedule", uds);
        request.setAttribute("logs", logs);
        
        this.forward(request, response, "getUserScheduledExportLog");
    }
    
    public void getCanUserAddMoreScheduledTasks(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        boolean canAddMore = false;
        
        //call the DAO method to see how many scheduled tasks a user has
        Long currentCount = this.getUserDocumentScheduleDAO().getUserDocumentScheduleCount(UserHelper.getUserName(request));
        int cap = this.getReportHelper().getAdvancedSchedulingUserTaskCap();
        
        if(currentCount < cap) {
            canAddMore = true;
        }
        
        //set up the return json
        StringWriter json = new StringWriter();
        JSONObject obj = new JSONObject();
        
        try {
            obj.put("canAddMore", canAddMore);
            obj.put("currentTaskCount", currentCount);
            obj.put("cap", cap);
            obj.write(json);
        } catch (JSONException ex) { 
            //do nothing
        }
        
        request.setAttribute("results", json.toString());
        
        this.forward(request, response, "resultsJSON");
    }
    
    private Date calculateEndDate(String frequency, int occurrences, Long weeklyRecurrenceWeek, Long weeklyRecurrenceDayOfWeek, Long monthlyRecurrenceMonth, String monthlyRecurrenceDay) {
        Date endDate = null;
        Date today = new Date();
            
        if(frequency.equals("daily")) {
            endDate = DateUtil.setTimeOnTheHour(today, 6); //set it to 6AM because the reports run starting at 1AM so we'll give it plenty of time to run before hitting the 6AM mark and not run
            endDate = DateUtil.addDays(endDate, occurrences);
        }

        if(frequency.equals("weekly")) {
            endDate = DateUtil.setTimeOnTheHour(today, 6); //set it to 6AM because the reports run starting at 1AM so we'll give it plenty of time to run before hitting the 6AM mark and not run
            
            //add weeks (days really) to the endDate
            endDate = DateUtil.addDays(endDate, ((occurrences * toIntExact(weeklyRecurrenceWeek * 7)) - (toIntExact(weeklyRecurrenceDayOfWeek) - DateUtil.getDayOfWeek(today))));
        }

        if(frequency.equals("monthly")) {
            endDate = DateUtil.setTimeOnTheHour(today, 6); //set it to 6AM because the reports run starting at 1AM so we'll give it plenty of time to run before hitting the 6AM mark and not run
            
            int dayOfMonthToRunOn = 1;
            int monthsToAdd = 0;
            int todaysDayOfTheMonth = DateUtil.getDayOfMonth(today);
            
            if(monthlyRecurrenceDay.equals("fifteenth")) {
                dayOfMonthToRunOn = 15;
            } 
            if(monthlyRecurrenceDay.equals("last")) {
                dayOfMonthToRunOn = DateUtil.getLastDayOfMonth(endDate);
            }
            
            //check if today is the same as 1, 15, or the last day of the month, if it is, then add a month to the endDate so it actually runs at least once
            if(todaysDayOfTheMonth == 1 || todaysDayOfTheMonth == 15 || todaysDayOfTheMonth == dayOfMonthToRunOn) {
                monthsToAdd = 1;
            }
            
            //add months
            endDate = DateUtil.addMonths(endDate, (occurrences * toIntExact(monthlyRecurrenceMonth)) + monthsToAdd);
            
            endDate = DateUtil.setDayOfMonth(endDate, dayOfMonthToRunOn);
        }
        
        return endDate;
    }    
    
    private String encodeToJSON(HashMap<String, Object> results, String draw) throws ParseException {
        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            List<List<String>> data = new ArrayList<List<String>>(); //overarching "data"
            SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd kk:mm:ss.S");
            SimpleDateFormat finalDate = new SimpleDateFormat("MM/dd/yyyy hh:mm a");
            SimpleDateFormat smallDate = new SimpleDateFormat("MM/dd/yyyy");
            List<UserDocumentSchedule> documents = (List<UserDocumentSchedule>) results.get("results");
            Long count = (Long) results.get("count");
            int rowNum = 1;
            
            for(UserDocumentSchedule doc : (List<UserDocumentSchedule>) documents) {
                List<String> dataItem = new ArrayList<String>();
                
                //checkbox
                dataItem.add("<input type='checkbox' name='userScheduledExportAction' class='userScheduledExportActionCheckbox' id='" + doc.getScheduleId().toString() + "' aria-label='Checkbox for " + doc.getUserDefinedReportName() + " in row " + rowNum + "' />");
                
                //dataItem.add("<a href='#' data-toggle='modal' data-scheduleid='" + doc.getScheduleId().toString() + "'>" + doc.getUserDefinedReportName() + "</a>");
                dataItem.add("<a href='ManageScheduledExports.do_sec?editUserScheduledExport=true&id=" + doc.getScheduleId().toString() + "' title='Edit Task'>" + doc.getUserDefinedReportName() + "</a> <a href='ManageScheduledExports.do_sec?getUserScheduledExportLog=true&id=" + doc.getScheduleId().toString() + "' title='View Task Log'>(View Log)</a>");
                dataItem.add(doc.getReportType());
                dataItem.add(doc.getFormat());
                Date date = dt.parse(doc.getCreationDate().toString());
                dataItem.add(finalDate.format(date).replaceFirst(" ", "<br />"));
                date = dt.parse(doc.getStartDate().toString());
                dataItem.add(smallDate.format(date));
                if(doc.getEndDate() != null) {
                    date = dt.parse(doc.getEndDate().toString());
                    dataItem.add(smallDate.format(date));
                } else {
                    dataItem.add("");
                }
                dataItem.add(doc.getFrequencyText());
                
                data.add(dataItem);
                
                rowNum++;
            }
            
            //Add in required Datables values
            obj.put("draw", draw);
            obj.put("recordsFiltered", count);
            obj.put("recordsTotal", count);
            obj.put("data", data);

            obj.write(json);

            return json.toString();
        } catch (JSONException ex) {
            Logger.getLogger(ScheduledExports.class.getName()).log(Level.SEVERE, null, ex);
        }
        return "";
    }
}
