<%--
    Document   : Data Quality Upload Transmission Log Report
    Created By : Ken So
--%>

<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date" %>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp" %>

<f:mainTemplate headerText="Data Quality Upload Transmission Log Report">
    <script language="JavaScript">
        $(document).ready(function () {
            $('#startDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#endDate').datepicker('min', curValue);
                }
            });
            
            $('#endDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                max: '<c:out value="${today}" />',
                onUpdate: function(curValue) {
                    $('#startDate').datepicker('max', curValue);
                }
            });
        
            VAP.common.setProcessingForDataTable('reportTable', 'divReportPanel');
            
            VAP.common.reportTable = $('#reportTable').DataTable({
                deferLoading: 0,
                destroy: true,
                ajax: { 
                    url: VAP.common.constructUrl("doSearch"),
                    dataSrc: function(json){
                        $('.dataTables_empty').remove();
                        return json.data;
                    },
                    error: function(o){                       
                        VAP.common.addSrollingToDataTable('reportTable');
                        VAP.common.handleErrorsInDataTable('reportTable', o, 12, 'divReportPanel');
                    }
                }, 
                info: true,
                paging: true,
                dom: "<'row'<'col-sm-12'l>>" +
                        "<'row'<'col-sm-6'i><'col-sm-6'p>>" +
                        "<'row'<'col-sm-12't<'pad-bottom-20'>r>>" +
                        "<'row'<'col-sm-5'i><'col-sm-7'p>>",
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25,
                columnDefs: [
                    {className: "text-center vertical-align-center", targets: [0,1,2,3,4,5,6,7,8,9,10,11,12]}
                ]
            });
            
            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('reportTable', VAP.common.reportTable);
                VAP.common.addSrollingToDataTable('reportTable');
            });
            
            $('#searchButton').on('click', function () {
                $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                
                var startDate = $('#startDate').val();
                var endDate = $('#endDate').val();

                var errorMessage = VAP.common.validateStartAndEndDate(startDate, endDate);
                
                if (errorMessage != '')
                {
                    //we have an error so show the error, and then don't send the ajax request
                    $('#errorMessage').html(errorMessage);
                    $('#errorMessage').show();

                    return false;
                } else {
                    //no error, let's make the call to populate the report
                    $('#results').show();
                    VAP.common.reportTable.ajax.url(VAP.common.constructUrl('doSearch')).load();
                }
            });

            $('#startDate,#endDate').on('change', function () {
                //if either date changes, hide the error message if its not already hidden
                $('#errorMessage').hide();
            });
            
            $('#results').hide();
            
            //set up JSON to pass to the Advanced Scheduling modal/functionality
            $('#btnScheduleExport').on('click', function(){
                //set up array of items to view at the top of the schedule export modal
                var viewParameters = [ 
                    {name : 'Event Type', value : $('#eventType option:selected').text()},
                    {name : 'Status', value : $('#status option:selected').text()},
                    {name : 'File Name', value : $('#fileName').val()}
                ];
                
                //set up array of items to pass as JSON in the schedule export modal form
                var parameters = {
                    'eventType' : $('#eventType').val(), 
                    'status': $('#status').val(), 
                    'fileName' : $('#fileName').val()
                };

                VAP.common.advancedSchedulingPopulateScheduleParameters(viewParameters, parameters, 'Data Quality Upload Transmission Log Report');
            });
        });
        
        VAP.common.constructUrl = function(methodName){
            var url = 'DqTransmissionLogReport.do_sec' +
                    '?' + methodName + '=true' +
                    '&startDate=' + $('#startDate').val() +
                    '&endDate=' + $('#endDate').val() +
                    '&eventType=' + $('#eventType').val() +
                    '&status=' + $('#status').val() +                              
                    '&filename=' + $('#fileName').val() +                            
                    '&recordsPerPage=' + $('#recordsPerPage').val() +
                    '&currentPage=' + $('#currentPage').val() +
                    '&sortBy=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][0] +
                    '&sortOrder=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][1];
            if (methodName === 'exportToCsv' || methodName === 'exportToExcel') {
                url = url + '&totalRows=' + VAP.common.reportTable.page.info().recordsTotal;
            }
            return url;
        };
        
        //Used by 'view' button in exchange final table column
        function documentView(documentIcn, documentUniqueID){
            $('#document-View-ICN').val(documentIcn);
            $('#document-View-Unique-Id').val(documentUniqueID);
            $('#documentViewForm').submit();
        }
    </script>
    
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="panel panel-default">
                <div class="panel-heading">
                    <div class="panel-title">
                        <a href="#" title="Search Criteria" aria-label="Search Criteria" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="receivedDocumentsReportSearchPanel"></i></a> Search Criteria
                    </div>
                </div>
                 <div class="panel-body" id="receivedDocumentsReportSearchPanel">
                    <div class="alert alert-info">
                        Fill in the fields below to narrow your search. All dates will be included if Start Date and End Date are left blank.
                    </div>
                    <div id="errorMessage" class="alert alert-warning pad-15 margin-bottom-10" style="display: none;" role="alert"></div>
                    <div id="userScheduledExportModalMessage" class="alert alert-dismissible" role="alert" style="display: none;">
                        <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                    </div>
                    <form name="receivedDocumentsReportForm" action="DqTransmissionLogReport.do_sec" method="POST" class="form-horizontal" role="form">
                        <input type="hidden" id="organizationName" name="organizationName" />
                        <input type="hidden" id="facilityName" name="facilityName" />
                        <div class="row">
                            <div class="col-xs-5 col-sm-5 col-md-5 col-lg-5">
                                <div class="form-group">
                                    <label for="eventType" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Event Type</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <%-- <select id="eventType" name="eventType" data-tooltip="This field allows for filtering on the event-type column in the Data Transmission Log database table, where event-type is the type of event that was logged." class="form-control"> --%>
                                        <select id="eventType" name="eventType" data-simpletooltip-text="This field allows for filtering on the event-type column in the Data Transmission Log database table, where event-type is the type of event that was logged." class="form-control js-simple-tooltip">
                                            <option value="" selected>ALL</option>
                                            <option value="DiameterHealth_UploadDataFile">DiameterHealth_UploadDataFile</option>
                                            <option value="DQZipFile_PackDataFileInside">DQZipFile_PackDataFileInside</option>
                                            <option value="DQZipFile_MoveZipFileForPickup">DQZipFile_MoveZipFileForPickup</option>
                                        </select>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="status" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Status</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">                                        
                                        <%-- <select id="status" name="status" data-tooltip="This field allows for filtering on the status column in the Data Transmission Log database table, where status indicates the success or failure state of the event that was logged." class="form-control"> --%>
                                        <select id="status" name="status" data-simpletooltip-text="This field allows for filtering on the status column in the Data Transmission Log database table, where status indicates the success or failure state of the event that was logged." class="form-control js-simple-tooltip">
                                            <option value="" selected>ALL</option>
                                            <option value="success">Success</option>
                                            <option value="failure">Failure</option>
                                        </select>                                        
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="fileName" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">File Name</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <%-- <input title="Enter File Name" id="fileName" name="fileName" type="text" maxlength="255" size="30" data-tooltip="This field allows for filtering on the filename column in the Data Transmission Log database table, where the eHX adapter file name of the document involved in the given event that was logged." class="form-control" /> --%>
                                        <input title="Enter File Name" id="fileName" name="fileName" type="text" maxlength="255" size="30" data-simpletooltip-text="This field allows for filtering on the filename column in the Data Transmission Log database table, where the eHX adapter file name of the document involved in the given event that was logged." class="form-control js-simple-tooltip" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="startDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Start Date</label>
                                    <%-- <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="This field indicates the beginning portion or start date of the search date range, and specifies that only log entries in the Data Transmission Log database table which have an event start date value equal to or after this date should be included in the search results.">
                                        <input type="text" name="startDate" id="startDate" class="form-control" value="<c:out value="${startDateDefault}" />" /> --%>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input type="text" name="startDate" id="startDate" class="form-control js-simple-tooltip" data-simpletooltip-text="This field indicates the beginning portion or start date of the search date range, and specifies that only log entries in the Data Transmission Log database table which have an event start date value equal to or after this date should be included in the search results." value="<c:out value="${startDateDefault}" />" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="endDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">End Date</label>
                                    <%-- <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="This field indicates the ending portion or end date of the search date range, and specifies that only log entries in the Data Transmission Log database table which have an event start date value equal to or earlier than this date should be included in the search results.">
                                        <input type="text" name="endDate" id="endDate" class="form-control" value="<c:out value="${endDateDefault}" />" /> --%>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input type="text" name="endDate" id="endDate" class="form-control js-simple-tooltip" data-simpletooltip-text="This field indicates the ending portion or end date of the search date range, and specifies that only log entries in the Data Transmission Log database table which have an event start date value equal to or earlier than this date should be included in the search results." value="<c:out value="${endDateDefault}" />" />
                                    </div>
                                </div>
                            </div>
                        </div>
                    </form>
                    <div class="row">
                        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                            <div class="form-group">
                                <div class="text-right">
                                    <cpp:userInRole role="VAPApplicationWebAdministrator">
                                        <button name="btnScheduleExport" id="btnScheduleExport" class="btn btn-default" data-toggle="modal" data-target="#scheduleExportModal" title="Schedule" aria-label="Schedule">Schedule</button>
                                    </cpp:userInRole>
                                    <input id="searchButton" name="search" title="Search" class="btn btn-primary" type="submit" value="Search" />
                                </div>
                            </div>
                        </div>
                    </div>
                 </div>
            </div>
        </div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="results">
        <div class="row">
            <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                <div class="panel panel-default" id="divReportPanel">
                    <div class="panel-heading">
                        <div class="panel-title">
                            <div class="pull-left">
                                <a href="#" title="Search Results" aria-label="Search Results" aria-expanded="true" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="receivedDocumentsReportResultsPanel"></i></a> Search Results
                            </div>
                            <div class="pull-right">
                                <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToExcel" title="Export to Excel" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to Excel</a>
                                <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToCsv" title="Export to CSV" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to CSV</a>
                            </div>
                            <div class="clearfix"></div>
                        </div>
                    </div>
                    <div class="panel-body" id="receivedDocumentsReportResultsPanel">
                        <form action="DqTransmissionLogReport.do_sec" method="POST" target="_blank" onsubmit="return doSubmit(this)" id="documentViewForm">
                            <input type="hidden" name="view" value="true">
                            <input type="hidden" name="documentIcn"  value="" id="document-View-ICN">
                            <input type="hidden" name="documentUniqueId" value="" id="document-View-Unique-Id">
                        </form>
                        <table id="reportTable" class="table table-striped" width="100%" cellspacing="0" role="presentation">
                            <thead>
                                <tr role="row">
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="DQ Transmission Id" id="DQTransmissionId">DQ Transmission Id</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="Batch Id" id="BatchId">Batch Id</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="Event Type" id="EventType">Event Type</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span data-tooltip="Start Time (CT)" id="StartTime">Start Time (CT)</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="EndTime" data-tooltip="End Time (CT)">End Time (CT)</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="HostName" data-tooltip="Host Name">Host Name</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="FileName" data-tooltip="File Name">File Name</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="FileType" data-tooltip="File Type">File Type</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="FileSize" data-tooltip="File Size">File Size</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="Destination" data-tooltip="Destination">Destination</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="Status" data-tooltip="Status" data-tooltip-position="left">Status</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="localDocId" data-tooltip="Local Doc Id" data-tooltip-position="left">Local Doc Id</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="remoteDocId" data-tooltip="Remote Doc Id" data-tooltip-position="left">Remote Doc Id</span></th> --%>
                                    <%-- <th role="columnheader" class="text-center"><span id="error" data-tooltip="Error" data-tooltip-position="left">Error</span></th> --%>
                                    <!--
                                    <th role="columnheader" class="text-center"><span id="viewHeader" data-tooltip="<c:out value="${descriptors['genericViewColumn']}" />" data-tooltip-position="left">View</span></th>
                                    -->
                                    
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="DQ Transmission Id"><span id="DQTransmissionId" data-simpletooltip-target="tableheader">DQ Transmission Id</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="Batch Id"><span id="BatchId" data-simpletooltip-target="tableheader">Batch Id</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="Event Type"><span id="EventType" data-simpletooltip-target="tableheader">Event Type</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="Start Time (CT)"><span id="StartTime" data-simpletooltip-target="tableheader">Start Time (CT)</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="End Time (CT)"><span id="EndTime" data-simpletooltip-target="tableheader">End Time (CT)</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="Host Name"><span id="HostName" data-simpletooltip-target="tableheader">Host Name</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-textp="File Name"><span id="FileName" data-simpletooltip-target="tableheader">File Name</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="File Type"><span id="FileType" data-simpletooltip-target="tableheader">File Type</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="File Size"><span id="FileSize" data-simpletooltip-target="tableheader">File Size</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="Destination"><span id="Destination" data-simpletooltip-target="tableheader">Destination</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="Status"><span id="Status" data-simpletooltip-target="tableheader">Status</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="Local Doc Id"><span id="localDocId" data-simpletooltip-target="tableheader">Local Doc Id</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="Remote Doc Id"><span id="remoteDocId" data-simpletooltip-target="tableheader">Remote Doc Id</span></th>
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="Error"><span id="error" data-simpletooltip-target="tableheader">Error</span></th>
                                    <!--
                                    <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['genericViewColumn']}" />"><span id="viewHeader" data-simpletooltip-target="tableheader">View</span></th>
                                    -->
                                </tr>
                            </thead>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    
    <%-- adding the advanced scheduling option to this report --%>
    <cpp:advancedScheduling />
</f:mainTemplate>