VAP = {};
VAP.common = {
    initVAP : function(){
        //polyfill for "startsWith" because IE doesn't support it. why? because IE...
        if (!String.prototype.startsWith) {
            String.prototype.startsWith = function(searchString, position) {
                position = position || 0;
                return this.indexOf(searchString, position) === position;
            };
        }
        /*$(document).on('click', '.vapCalendar', function() {
            $(this).parent().parent().find('input').focus();
        });*/
        
        //used on date pickers so they won't show the input in red (error) when the calendar icon/button is clicked
        $('.vap-datepicker-input').on( "ab.datepicker.opening", function(){
            // fired when the date picker is opening
            $(this).removeAttr('aria-invalid');
            $(this).parents('.form-group').removeClass('has-error');
        });

        $('.vapFormReset').click(function( event ) {
            $(this).closest('form')[0].reset();
        });
        
        //does a direct download of anything that uses the contructUrl method (initially used for direct excel downloads)
        $('.directDownload').on('click', function(){
            var baseUrl = $(this).data('method');
            
            window.location.href = VAP.common.constructUrl(baseUrl);
        });

        $(document).on('click', '.allableCheckbox', function () {
            if ($(this).val() === 'ALL') {
                if ($('.allableCheckbox[value=ALL]').is(':checked')) {
                    $('.allableCheckbox').each(function () {
                        $(this).prop("checked", false);
                    });
                    $('.allableCheckbox[value=ALL]').prop("checked", true);
                }
            } else {
                var nonAllChecked = false;
                $('.allableCheckbox').each(function () {
                    nonAllChecked |= $(this).prop("checked");
                });
                $('.allableCheckbox[value=ALL]').prop("checked", !nonAllChecked);
            }
        });
        
        //for all of the tags with a data-tooltip attribute, add the appropriate tooltip for it
        if ($("[data-tooltip]") !== null) {
            $("[data-tooltip]").each(function(){
                var element = $(this);
                var tooltipText = element.data('tooltip'); 

                if(element.is("[data-tooltip-position]")) {
                    var position = element.data('tooltip-position');
                    if(position === 'left') { //so far we only have left alignment as an option other than the default
                        var pos = {
                            my: 'center right',
                            at: 'center left'
                        };
                    }
                    if(position === 'scroll2') { //so far we only have left alignment as an option other than the default
                        var pos = {
                            my: 'center left',
                            at: 'center right',
                            adjust:{x: 427}
                        };
                    }
                    
                } else { //this is the default
                    var pos = { 
                        my: 'center left',
                        at: 'center right'
                    };
                }

                element.qtip($.extend(VAP.common.tipOptions, {
                    content: tooltipText,
                    position: pos,
                    prerender: true
                }));
            });
        };
        
        //handle modals with bootstrap now for csvButton and excelButton
        $('#confirmExportModal').on('show.bs.modal', function (event) {
            var link = $(event.relatedTarget) // link that triggered the modal
            var methodOfExport = link.data('method');
            var textToUse = '';
            
            if (VAP.common.reportTable.page.info().recordsTotal < VAP.common.scheduledExportThreshold) {
                textToUse = 'The information that you are about to download is for official use only and is considered protected health information and must be appropriately protected using encryption per VA policy.';
            } else {
                if(methodOfExport === 'exportToExcel') {
                    textToUse = '<p>The information that you are about to download is for official use only and is considered protected health information and must be appropriately protected using encryption per VA policy.</p><p>Click the Export button below to generate the Excel export. This will appear on the Scheduled Exports page when completed.</p>';
                } else {
                    textToUse = '<p>The information that you are about to download is for official use only and is considered protected health information and must be appropriately protected using encryption per VA policy.</p><p>Click the Export button below to generate the CSV export. This will appear on the Scheduled Exports page when completed.</p>';
                }
            }
            
            var modal = $(this);
            
            modal.find('.modal-body').html(textToUse); //set the text into the modal body
            modal.find('.modal-footer #confirmExportModalExport').attr('data-base-method', methodOfExport); //add the method into the footer "export" button so the click handler for this button knows what method to use for a download
            modal.find('.modal-footer #confirmExportModalExport').attr('data-validate-ssns', !!link.attr('data-validate-ssns'));
        });
        
        $('#confirmExportModalExport').on('click', function(){
            var mustValidateSSNsBeforeExport = $(this).data('validate-ssns');
            var baseUrl = $(this).attr('data-base-method'); // getting the baseUrl using attr because this: $(this).data('base-method') does not seem to work
            
            if(VAP.common.reportTable.page.info().recordsTotal < VAP.common.scheduledExportThreshold){
                if(mustValidateSSNsBeforeExport) {
                    if(VAP.common.validateSsns()){
                        window.location.href = VAP.common.constructUrl(baseUrl);
                    }
                } else {
                    window.location.href = VAP.common.constructUrl(baseUrl);
                }

            } else {
                if(mustValidateSSNsBeforeExport){
                    if(VAP.common.validateSsns()){
                        $.ajax(VAP.common.constructUrl(baseUrl));
                    }
                } else {
                    $.ajax(VAP.common.constructUrl(baseUrl));
                }
            }
        });
        
        $('#pdfLink').on('click', function() {
            var b64Data = document.getElementById('pdfString').value;
            var contentType = 'application/pdf';
            var sliceSize = 512;
            b64Data = b64Data.replace(/^[^,]+,/, '');
            b64Data = b64Data.replace(/\s/g, '');
            var byteCharacters = window.atob(b64Data);
            var byteArrays = [];

            for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
                var slice = byteCharacters.slice(offset, offset + sliceSize);

                var byteNumbers = new Array(slice.length);
                for (var i = 0; i < slice.length; i++) {
                    byteNumbers[i] = slice.charCodeAt(i);
                }

                var byteArray = new Uint8Array(byteNumbers);

                byteArrays.push(byteArray);
            }

            var blob = new Blob(byteArrays, {type: contentType});

            if (window.navigator.msSaveOrOpenBlob) {
                window.navigator.msSaveBlob(blob, "attachment.pdf");
            } else {
                var elem = window.document.createElement('a');
                elem.href = window.URL.createObjectURL(blob);
                elem.download = "attachment.pdf";        
                document.body.appendChild(elem);
                elem.click();        
                document.body.removeChild(elem);
                window.URL.revokeObjectURL();
            }
        });
        
        //global error handling for ajax requests that occur after user is logged out (usually do to a session time out/no header in the request)
        $(document).ajaxError(function (e, jqXHR, ajaxSettings, thrownError) {
            // if 901 - user does not have an active session/no header in the request, so redirect to the patient search page and the error handler will do the rest
            if (jqXHR.status == '901') {
                window.location.href = 'PatientSearch.do_sec';
            }
        });
        
        //toggle of any vap panel
        $('.vap-panel-hide-show').on('click', function(event){
            var me = $(this); //the href tag the class is on
            var icon = me.find('i');
            var thePanelBody = $('#' + icon.data('panel-body'));
            if(icon.hasClass('fa-minus')) {
                //the panel is open, so swap the fa-minus for fa-plus and collapse the panel
                icon.removeClass('fa-minus').addClass('fa-plus');
                thePanelBody.removeClass('collapse.in').addClass('collapse');
                if (me.attr('aria-label').startsWith('Close')) {
                    me.attr('aria-label', me.attr('aria-label').replace('Close', 'Open'));
                }
                if (me.attr('title').startsWith('Close')) {
                    me.attr('title', me.attr('title').replace('Close', 'Open'));
                }
                if (me.attr('aria-expanded').startsWith('true')) {
                    me.attr('aria-expanded', me.attr('aria-expanded').replace('true', 'false'));
                }
            } else {
                //the panel is closed, so swap the fa-plus for fa-minus and show the panel
                icon.removeClass('fa-plus').addClass('fa-minus');
                thePanelBody.removeClass('collapse').addClass('collapse.in');
                if (me.attr('aria-label').startsWith('Open')) {
                    me.attr('aria-label', me.attr('aria-label').replace('Open', 'Close'));
                }
                if (me.attr('title').startsWith('Open')) {
                    me.attr('title', me.attr('title').replace('Open', 'Close'));
                }
                if (me.attr('aria-expanded').startsWith('false')) {
                    me.attr('aria-expanded', me.attr('aria-expanded').replace('false', 'true'));
                }
            }
        });
        
        //handler for log out link
        $('#confirmLogoutModalLogout').on('click', function(){
            window.location.href = 'Logout.do_sec';
        });
        
        //global overrides for blockUI
        $.blockUI.defaults.overlayCSS.opacity = 0.8;
        $.blockUI.defaults.overlayCSS.backgroundColor = '#CFD0D3';
        $.blockUI.defaults.message = '<strong style="opacity: 0.5;">Processing...</strong>';
        $.blockUI.defaults.css.top = '26px';
        $.blockUI.defaults.centerY = false;
        $.blockUI.defaults.css.border = 'none';
        $.blockUI.defaults.css.backgroundColor = '(207, 208, 211, 0.8)';
        
        //global handler for ajax calls to show the loading gif
        $(document).ajaxSend(function (event, request, settings) {   
            if(typeof settings.spinnerTargetElement != 'undefined') {
                var elementForSpinner = settings.spinnerTargetElement;
                
                //add the spinner
                if(elementForSpinner.length > 0){
                    $('#'+elementForSpinner).addClass('vap-loading');
                } 
            }
            
            if(typeof settings.headers != 'undefined' && settings.headers.skipLoadingAlert == 'true') { 
                // this header is there because either it is a regular ajax call OR we don't want to pop the alerts
            } else {
                //pop the alert message for the screen readers
                $('#reportTable_info').attr('title', 'Generating results, please wait');
                VAP.common.fireAriaAlert('Generating results, please wait');
            }
        }).ajaxComplete(function (event, request, settings) {
            if(typeof settings.spinnerTargetElement != 'undefined') {
                var elementForSpinner = settings.spinnerTargetElement;
                
                //remove the spinner
                if(elementForSpinner.length > 0){
                    $('#'+elementForSpinner).removeClass('vap-loading');
                }
            }
            
            if(typeof settings.headers != 'undefined' && settings.headers.skipLoadingAlert == 'true') { 
                // this header is there because either it is a regular ajax call OR we don't want to pop the alerts
            } else {
                //pop the alert message for the screen readers
                $('#reportTable_info').attr('title', 'Results have been generated');
                VAP.common.fireAriaAlert('Results have been generated');
            }
        });
        
        //global options for DataTables
        $.extend( true, $.fn.dataTable.defaults, {
            language: {
                emptyTable: "",
                loadingRecords: "",
                zeroRecords: ""
            },
            drawCallback: function(settings) {
                var api = this.api();
                var tableId = $(this).selector;
                if(api.rows( {page:'current'} ).data().length < 1) {
                    //we have no rows so we need to display the no records found
                    if(VAP.common.reportNoRecordsMessage.length > 0) { //look for this global variable that can be set to show other messages other than the default
                        $(tableId + ' .dataTables_empty').html(VAP.common.reportNoRecordsMessage);
                    } else {
                        $(tableId + ' .dataTables_empty').html('No records found.');
                    }
                    
                }
            }
        });
        
        //override for DataTables to NOT throw or do anything on an error. we have the error handler for this (see handleErrorsInDataTable below)
        $.extend(true, $.fn.dataTable.ext, {
           errMode: 'none' 
        });
        
        //set up global styling for toast messages
        $.toast.options = {
            loaderBg : '#9EC600',
            showHideTransition : 'slide',
            hideAfter : 6200, 
            stack : 5, 
            position : {
                bottom : 45,
                left : 5
            }
        };
    },
    
    fireAriaAlert : function(message) {
        var newAlert = document.createElement("div"); /* using the native js because it's faster */
        newAlert.setAttribute("role", "alert");
        newAlert.setAttribute("id", "JAWSCompliantAlert");
        newAlert.setAttribute("class","sr-only");
        var msg = document.createTextNode(message);
        newAlert.appendChild(msg);
        document.body.appendChild(newAlert);
        
        //wait five seconds and then remove this newly created alert div
        setTimeout(function () {
            $('#JAWSCompliantAlert').addClass('hidden');
            $('#JAWSCompliantAlert').remove();
        }, 5000);
    },
    
    escapeHTML : function(unsafe) {
        return unsafe
            .replace(/&/g, "&amp;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/"/g, "&quot;")
            .replace(/'/g, "&#039;");
     },
    
    //DataTables specific overrides/functions

    addAriaToDataTable : function(tableId, dataTableVariable) {
        var $tbody = $('#' + tableId + '').find("tbody");
        $tbody.find("th").attr("role", "rowheader");
        $tbody.find("td").attr("role", "gridcell");

        if ( !dataTableVariable.data().any() ) {
            //we have no records so we have to add a role="row" to the returned tr for aria compliance
            $tbody.find("tr").attr("role", "row");
        }
        
        //add the current state of ascending or descending information to the aria-label tag (if it exists)
        var headerWithSort = $('#' + tableId + '').find('th[aria-sort]');
        
        if(headerWithSort.length != 0) {
            //okay we found a column heading that this report is sorted by, so let's add to the aria-label the current state
            var currentSortOrder = headerWithSort.attr('aria-sort');

            var currentAriaLabelText = headerWithSort.attr('aria-label');
            headerWithSort.attr('aria-label', currentAriaLabelText + ' Currently sorted ' + currentSortOrder + '.');
        }
    },
    
    addSrollingToDataTable : function(tableId) {
        
        var tableContentRow = $("#"+tableId+"_wrapper").find("table");
        tableContentRow = tableContentRow.parent().parent();
        tableContentRow.addClass("HorizontalScrollDiv");
        tableContentRow.attr('id',tableId+'BottomScroll');
        tableContentRow.removeClass("row");
        
        tableContentRow.prev(".extraScrollBar").remove();
        tableContentRow.before("<div id='"+tableId+"TopScroll' class='extraScrollBar'><div class='col-sm-12'><div class='innerScrollDiv'></div></div></div>");
        //$("#"+tableId+"TopScroll").css({'width':tableContentRow.width()+'px'});
        
        //add width to inner div 
        $("#"+tableId+"TopScroll"+" div .innerScrollDiv").css({'width':($("#"+tableId).width())+'px'});
        
        //link the two scroll bars together
        $("#"+tableId+"TopScroll").on('scroll', function () {
            $("#"+tableId+"BottomScroll").scrollLeft($(this).scrollLeft());
        });
        
        $("#"+tableId+"BottomScroll").on('scroll', function () {
            $("#"+tableId+"TopScroll").scrollLeft($(this).scrollLeft());
        });
    },
    
    setProcessingForDataTable : function(tableId, divToBlock) {
        $('#' + tableId).on( 'processing.dt', function (e, settings, processing) {
            $('#' + tableId + '_processing').css('display', 'none');
            if(processing) {
                $('#' + divToBlock).block();
            } else {
                $('#' + divToBlock).unblock();
            }
        }).dataTable();
    },

    handleErrorsInDataTable : function(tableId, responseObject, colsToSpan, elementToUnblock) {
        $('#' + tableId + '_processing').hide();
        $('.dataTables_empty').remove();
        var json = responseObject.responseJSON;

        //handle session expirations
        if(responseObject.status == '901') {
            window.location.href = 'Login.do';
        }
        
        //unblock the table/element because the processing might still be going if we get an error
        if (typeof elementToUnblock === "undefined") { //elementToUnblock was added later, so if it is undefined, we use the tableId which is what this used originally
            $('#' + tableId).unblock();
        } else {
            $('#' + elementToUnblock).unblock();
        }
        
        if(json.errorData){
            var message = json.errorData.customMessage.length > 0 ? json.errorData.customMessage : 'An Error Occurred';
            $('#' + tableId + ' tbody').html('<tr class="odd" role="row"><td valign="top" colspan="' + colsToSpan + '" class="dataTables_empty text-center" role="gridcell">' + message + '</td></tr>');
        } else {
            $('#' + tableId + ' tbody').html('<tr class="odd" role="row"><td valign="top" colspan="' + colsToSpan + '" class="dataTables_empty text-center" role="gridcell">An Error Occurred</td></tr>');
        }
    },

    //called on the success of the last JSChart created on a page (as they are currently chained together)
    addAriaToJSCharts : function(){
        $('.chartjs-hidden-iframe').each(function(i, obj) {
            var titleToSet = $(obj).siblings("canvas").data('title');
            $(obj).attr("title", titleToSet);
        });
    },

    //common method to validate a start and end date. returns an error message if one is found or returns a blank one if none are found. validates dates in mm/dd/yyyy format
    validateStartAndEndDate : function(startDate, endDate, futureDatesEnabled) {
        var errorMessage = '';
        var today = moment();
        var re = /^(\d{1,2})\/(\d{1,2})\/(\d{4})$/;

        if (typeof futureDatesEnabled === 'undefined') {
            futureDatesEnabled = false; //default this to false if its not passed in as it is an optional param
        }

        if ((startDate === '' || (moment(startDate, 'MM/DD/YYYY', true).isValid() && startDate.match(re))) && ((endDate === '' || (moment(endDate, 'MM/DD/YYYY', true).isValid() && endDate.match(re))))) {
            var startDateToUse = moment(startDate, 'MM/DD/YYYY');
            var endDateToUse = moment(endDate, 'MM/DD/YYYY');

            if (!futureDatesEnabled) { //if the future dates are NOT enabled (default) then we check for start and end dates that need to be prior or equal to today
                if (startDate !== '') {
                    if (startDateToUse.isAfter(today)) {
                        errorMessage = 'Start date must be prior or equal to today.';
                    }
                }
                if (endDate !== '') {
                    if (endDateToUse.isAfter(today)) {
                        errorMessage = 'End date must be prior or equal to today.';
                    }
                }
            }
            if (startDate !== '' && endDate !== '') {
                if (startDateToUse.isAfter(endDateToUse)) {
                    errorMessage = 'Start date must be before or equal to end date.';
                }
            }
        } else {
            errorMessage = 'Start date and end date must be valid dates in the mm/dd/yyyy format.';
        }

        return errorMessage;
    },
    
    validateStartAndEndDateWithTime : function(startDate, endDate, futureDatesEnabled, maxFutureDate) {
        var errorMessage = '';
        var today = moment();
        
        if (typeof futureDatesEnabled === 'undefined') {
            futureDatesEnabled = false; //default this to false if its not passed in as it is an optional param
        }
        
        if(typeof maxFutureDate === 'undefined') {
            var checkForMaxFutureDate = false;
        } else {
            var checkForMaxFutureDate = true;
        }
        
        if ((startDate === '' || (moment(startDate, 'MM/DD/YYYY hh:mm A', true).isValid() )) && ((endDate === '' || moment(endDate, 'MM/DD/YYYY hh:mm A', true).isValid() ))) {
            var startDateToUse = moment(startDate, 'MM/DD/YYYY hh:mm A');
            var endDateToUse = moment(endDate, 'MM/DD/YYYY hh:mm A');
            
            if (!futureDatesEnabled) { //if the future dates are NOT enabled (default) then we check for start and end dates that need to be prior or equal to today
                if (startDate !== '') {
                    if (startDateToUse.isAfter(today)) {
                        errorMessage = 'Start date must be prior or equal to today.';
                    }
                }
                if (endDate !== '') {
                    if (endDateToUse.isAfter(today)) {
                        errorMessage = 'End date must be prior or equal to today.';
                    }
                }
            }
            
            if(checkForMaxFutureDate) { //a max future date has been specified so we need to check that the start and end dates are NOT past that date. time can be anything, that's why we have the 11:59 PM specified below for the maxFutureDateToUse
                var maxFutureDateToUse = moment(maxFutureDate + ' 11:59 PM', 'MM/DD/YYYY hh:mm A');
                
                if(startDateToUse.isAfter(maxFutureDateToUse) || endDateToUse.isAfter(maxFutureDateToUse)) {
                    errorMessage = 'Start date and end date must be before or equal to ' + maxFutureDate + '.';
                }
            }
            
            if (startDate !== '' && endDate !== '') {
                if (startDateToUse.isAfter(endDateToUse)) {
                    errorMessage = 'Start date must be before or equal to end date.';
                }
            }
        } else {
            errorMessage = 'Start date and end date must be valid dates in the mm/dd/yyyy format.';
        }
        
        return errorMessage;
    },
    
    validateSsns : function(field) {
        var sansWhiteSpace = field.val().replace(/ /g, '');
        if (sansWhiteSpace !== null) {
            if (/^\d{9}(,\d{9})*$/.test(sansWhiteSpace) || sansWhiteSpace === '') {
                field.val(sansWhiteSpace);
                return true;
            } else {
                return false;
            }
        }
    },
    
    isDate : function(input) {
        if (!/^\d{2}\/\d{2}\/\d{4}$/.test(input)) {
            return false;
        } else {
            var monthfield = input.split("/")[0];
            var dayfield = input.split("/")[1];
            var yearfield = input.split("/")[2];

            var dayobj = new Date(yearfield, monthfield - 1, dayfield);
            if ((dayobj.getMonth() + 1 != monthfield) || (dayobj.getDate() != dayfield) || (dayobj.getFullYear() != yearfield)) {
                return false;
            } else {
                return true;
            }
        }
    },
    
    constructUrl : function() {
        //this function must be overridden on a report page to be used properly
    },
    
    //global options for the tool tips (descriptors in the session scope)
    tipOptions : {
        position : {
            my: 'center left',
            at: 'center right'
        },
        show : {
            delay: 700
        }
    },
    
    adjustIframeHeightOnLoad : function() {	
        $("#ccdaframe1").style.height = $("#ccdaframe1").contentWindow.document.body.scrollHeight + "px"; 
        $("#ccdaframe1").contentWindow.document.body.style.paddingTop = "0px";
    },
    
    isInternetExplorer : function() {
        var ua = window.navigator.userAgent;
        var msie = ua.indexOf("MSIE ");
        var isIE = false;

        if (msie > 0 || !!navigator.userAgent.match(/Trident.*rv\:11\./))  // If Internet Explorer, return version number
        {
            isIE = true;
        }
        
        return isIE;
    },
    
    initToastForInternetExplorer : function() {
        if(VAP.common.isInternetExplorer) {
            //pop a "hidden" toast so it gets initialized in IE. otherwise, the first time you pop a toast, it won't get read by the screen readers
            $.toast({
                text: "", 
                showHideTransition: 'plain',
                allowToastClose: false, 
                hideAfter: 1, 
                position : {
                    bottom : 45,
                    left : 5
                },
                bgColor: '#ffffff',  
                textColor: '#ffffff',  
                loader: false
            });
        }
    },
    
    //advanced scheduling methods
    advancedSchedulingPopulateScheduleParameters : function(viewParameters, parameters, reportType) {
        //loop over the view parameters to create the nice view of parameters at the top of the schedule export modal
        var parametersView = '';
        
        $.each(viewParameters, function(index, obj){
            if($.isArray(obj.value)) { //if an object value is an array (like a bunch of values from a multi-select) then wrap a div around it and give it some space and a scrollbar
                parametersView += '<strong>' + obj.name + ' : </strong><div style="overflow: auto; max-height: 100px;">' + obj.value + '<div><br />';
            } else {
                parametersView += '<strong>' + obj.name + ' : </strong>' + obj.value + '<br />';
            }
            
        });
        
        //add the view parameters to the top div in the schedule export modal
        $('#scheduledExportParametersView').html(parametersView);
        
        //create the json from all the parameters and put it into the hidden scheduledExportParametersJSON and scheduledExportViewParametersJSON fields in the schedule export form
        $('#scheduledExportParametersJSON').val(JSON.stringify(parameters));
        $('#scheduledExportViewParametersJSON').val(JSON.stringify(viewParameters));
        
        //set the reportType hidden input
        $('#reportType').val(reportType);
        
        //alter the modal title
        $('#scheduleExportModalLabel').html('Advanced Scheduling: ' + reportType);
    },
    
    advancedSchedulingValidateFrequency : function() {
        var frequencySelected = $('input[name=frequency]:checked').val();
        var re = new RegExp("^[1-9]\d?$");

        //clear all errors for frequency
        $('#frequencyWeekly').removeClass('has-error has-danger text-danger');
        $('#weeklyRecurrences').removeClass('has-error has-danger');
        $('#frequencyMonthly').removeClass('has-error has-danger text-danger');
        $('#monthlyOccurrences').removeClass('has-error has-danger');

        //logic to validate frequency

        if(frequencySelected === 'weekly') {
            var weeklyRec = $('#weeklyRecurrences').val();

            if (!re.test(weeklyRec) || !weeklyRec.length > 0 || !(weeklyRec >= 1 && weeklyRec <= 6)) { //has to be a number and is required and must be in the range of 1-6 
                $('#frequencyWeekly').addClass('has-error has-danger text-danger');
                $('#weeklyRecurrences').addClass('has-error has-danger');
            }
        }

        if(frequencySelected === 'monthly') {
            var monthlyRec = $('#monthlyOccurrences').val();

            if (!re.test(monthlyRec) || !monthlyRec.length > 0 || !(monthlyRec >= 1 && monthlyRec <= 6)) { //has to be a number and is required and must be in the range of 1-6
                $('#frequencyMonthly').addClass('has-error has-danger text-danger');
                $('#monthlyOccurrences').addClass('has-error has-danger');
            }
        }
    },
            
    advancedSchedulingValidateEndDate : function() {
        var endDateSelected = $('input[name=endDateSelection]:checked').val();
        var re = new RegExp("^([1-9][0-9]?)$");

        //clear all errors for endDate
        $('#divEndDateEndAfter').removeClass('has-error has-danger');
        $('#endAfterOccurrences').removeClass('has-error has-danger');
        $('#divEndDateEndBy').removeClass('has-error has-danger');
        $('#scheduledEndDate').removeClass('has-error has-danger');
        $('#spanEndDateOccurrences').removeClass('text-danger');

        if(endDateSelected === 'endAfter') {
            var endDateOcc = $('#endAfterOccurrences').val();
            
            if (!re.test(endDateOcc) || !(endDateOcc.length > 0) || !(endDateOcc >= 1 && endDateOcc <= 30)) { //has to be a number and is required and must be in the range of 1-30
                $('#divEndDateEndAfter').addClass('has-error has-danger');
                $('#endAfterOccurrences').addClass('has-error has-danger');
                $('#spanEndDateOccurrences').addClass('text-danger');
            }
        }

        if(endDateSelected === 'endBy') {
            var today = moment();
            var dateToValidate = moment($('#scheduledEndDate').val(), 'MM/DD/YYYY');
            var sixMonthsFromNow = moment().add('months', 6);
            
            if(!moment($('#scheduledEndDate').val(), 'MM/DD/YYYY', true).isValid() || !dateToValidate.isAfter(today) || dateToValidate.isAfter(sixMonthsFromNow)) {    
                $('#divEndDateEndBy').addClass('has-error has-danger');
                $('#scheduledEndDate').addClass('has-error has-danger');
            }
        }
    },
    
    advancedSchedulingGetUserTaskInfo : function() {
        // make a call out to get advanced scheduling task info (cap, user's current count, if they can add more). use this with a promise so the data comes back before you need to use it...not after
        return $.ajax({
            url: "ManageScheduledExports.do_sec?getCanUserAddMoreScheduledTasks", 
            headers: { 'skipLoadingAlert' : 'true' },
            success : function (result) {
                var info = $.parseJSON(result);

                //set global variables to be used later to check if a user can submit the form (if they are under the limit)
                VAP.common.advancedScheduledTaskCap = info.cap;
                VAP.common.advancedScheduledTaskUserCurrentTaskCount = info.currentTaskCount;
                VAP.common.advancedScheduledTaskUserCanAddMore = info.canAddMore;
            }, 
            error: function(result) {
               //do nothing 
            }
        }); 
    }
};
